@@ -11,19 +11,20 @@
  * Copyright IBM Corp. 2015
  */
 
-#include <assert.h>
 #include <errno.h>
 #include <pthread.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <syslog.h>
 
+#include "fips.h"
 #include "s390_crypto.h"
 #include "s390_drbg.h"
 #include "s390_sha.h"
 
-#define NDEBUG	/* turns off assertions */
+#define MAX_NO_OF_BYTES	(255 * DRBG_OUT_LEN)	/* limit for hash_df */
 
 /*
  * Test DRBG mechanisms
@@ -42,6 +43,9 @@ static int test_reseed_error_handling(ica_drbg_mech_t *mech);
 
 static int test_generate_error_handling(ica_drbg_mech_t *mech);
 
+static int set_error_state(ica_drbg_mech_t *mech,
+			   int error);
+
 /*
  * DRBG mechanism list. Add new DRBG mechanism here:
  */
@@ -158,7 +162,7 @@ int drbg_instantiate(ica_drbg_t **sh,
 				   entropy_len, nonce, nonce_len);
 	if(status){
 		if(0 > status)
-			mech->error_state = status;
+			set_error_state(mech, status);
 		goto _exit_;
 	}
 
@@ -242,11 +246,11 @@ int drbg_reseed(ica_drbg_t *sh,
 	}
 
 	/* steps 6 and 7 */
-	assert(!pthread_mutex_lock(&sh->lock));
+	pthread_mutex_lock(&sh->lock);
 	status = sh->mech->reseed(sh->ws, add, add_len, entropy, entropy_len);
-	assert(!pthread_mutex_unlock(&sh->lock));
+	pthread_mutex_unlock(&sh->lock);
 	if(0 > status)
-		sh->mech->error_state = status;
+		set_error_state(sh->mech, status);
 
 	/* step 8 */
 _exit_:
@@ -302,14 +306,14 @@ int drbg_generate(ica_drbg_t *sh,
 
 	/* step 7 */
 _reseed_req_:
-	assert(!pthread_mutex_lock(&sh->lock));
+	pthread_mutex_lock(&sh->lock);
 	if(pr || reseed_required){
 		/* steps 7.1 and 7.3 */
 		status = drbg_reseed(sh, pr, add, add_len, test_mode,
 				     test_entropy, test_entropy_len);
 		/* step 7.2 */
 		if(status){
-			assert(!pthread_mutex_unlock(&sh->lock));
+			pthread_mutex_unlock(&sh->lock);
 			return status;	/* return reseed status */
 		}
 		/* step 7.4 */
@@ -321,7 +325,7 @@ _reseed_req_:
 
 	/* steps 8 and 10 */
 	status = sh->mech->generate(sh->ws, add, add_len, prnd, prnd_len);
-	assert(!pthread_mutex_unlock(&sh->lock));
+	pthread_mutex_unlock(&sh->lock);
 
 	/* step 9 */
 	if(DRBG_RESEED_REQUIRED == status){
@@ -334,7 +338,7 @@ _reseed_req_:
 		goto _reseed_req_;
 	}
 	else if(0 > status)
-		sh->mech->error_state = status;
+		set_error_state(sh->mech, status);
 
 	/* step 11 */
 	return status;
@@ -355,15 +359,15 @@ int drbg_uninstantiate(ica_drbg_t **sh,
 		return status;
 
 	/* step 2 */
-	assert(!pthread_mutex_lock(&(*sh)->lock));
+	pthread_mutex_lock(&(*sh)->lock);
 	status = (*sh)->mech->uninstantiate(&(*sh)->ws, test_mode);
 	if(status){
 		if(0 > status)
-			(*sh)->mech->error_state = status;
+			set_error_state((*sh)->mech, status);
 		return status;	/* return uninstantiate status */
 	}
-	assert(!pthread_mutex_unlock(&(*sh)->lock));
-	assert(!pthread_mutex_destroy(&(*sh)->lock));
+	pthread_mutex_unlock(&(*sh)->lock);
+	pthread_mutex_destroy(&(*sh)->lock);
 	drbg_zmem(*sh, sizeof(ica_drbg_t));
 	if(test_mode)
 		status = drbg_check_zmem(*sh, sizeof(ica_drbg_t));
@@ -379,7 +383,7 @@ int drbg_health_test(const void *func,
 		     bool pr,
 		     ica_drbg_mech_t *mech)
 {
-	int status;
+	int status, i;
 	const int SEC[] = {DRBG_SEC_112, DRBG_SEC_128, DRBG_SEC_192,
 			   DRBG_SEC_256};
 
@@ -392,19 +396,19 @@ int drbg_health_test(const void *func,
 		status = mech->health_test(drbg_instantiate, sec, pr);
 		if(status){
 			if(0 > status)
-				mech->error_state = status;
+				set_error_state(mech, status);
 			return status;
 		}
 
 		/* Error handling test. */
 		status = test_instantiate_error_handling(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, DRBG_HEALTH_TEST_FAIL);
 
 		/* Uninstantiate test. */
 		status = test_uninstantiate(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, DRBG_HEALTH_TEST_FAIL);
 
 		return 0;
 	}
@@ -413,26 +417,25 @@ int drbg_health_test(const void *func,
 		status = mech->health_test(drbg_reseed, sec, pr);
 		if(status){
 			if(0 > status)
-				mech->error_state = status;
+				return set_error_state(mech, status);
 			return status;
 		}
 
 		/* Error handling test. */
 		status = test_reseed_error_handling(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, status);
 
 		/* Uninstantiate test. */
 		status = test_uninstantiate(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, status);
 
 		return 0;
 	}
 	else if(drbg_generate == func){
 		/* Test vectors: test all combinations sec, pr supp, pr req */
-		int i = 0;
-		for(; i < sizeof(SEC) / sizeof(SEC[0]); i++){
+		for(i = 0; i < sizeof(SEC) / sizeof(SEC[0]); i++){
 			if(SEC[i] > mech->highest_supp_sec)
 				break;
 
@@ -440,7 +443,7 @@ int drbg_health_test(const void *func,
 						   false);
 			if(status){
 				if(0 > status)
-					mech->error_state = status;
+					set_error_state(mech, status);
 				return status;
 			}
 
@@ -448,7 +451,7 @@ int drbg_health_test(const void *func,
 						   true);
 			if(status){
 				if(0 > status)
-					mech->error_state = status;
+					set_error_state(mech, status);
 				return status;
 			}
 		}
@@ -456,12 +459,12 @@ int drbg_health_test(const void *func,
 		/* Error handling test. */
 		status = test_generate_error_handling(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, status);
 
 		/* Uninstantiate test.*/
 		status = test_uninstantiate(mech);
 		if(status)
-			return mech->error_state = DRBG_HEALTH_TEST_FAIL;
+			return set_error_state(mech, status);
 
 		return 0;
 	}
@@ -514,6 +517,7 @@ int drbg_get_entropy_input(bool pr,
 int drbg_get_nonce(unsigned char *nonce,
 		   size_t nonce_len)
 {
+	size_t i;
 	static uint16_t ctr;
 
 	/* The buffer for nonce must hold a 16 byte timestamp. */
@@ -534,8 +538,7 @@ int drbg_get_nonce(unsigned char *nonce,
 	pthread_t thread_id = pthread_self();
 
 	/* Store bytewise XOR of the thread id in first byte. */
-	int i = 0;
-	for(; i < sizeof(thread_id); i++)
+	for(i = 0; i < sizeof(thread_id); i++)
 		*nonce ^= *((unsigned char *)&thread_id + i);
 
 	/* Store counter in the last two bytes. Since TOD clock is thread-save,
@@ -551,9 +554,13 @@ int drbg_hash_df(const unsigned char *input,
 		 unsigned char *req_bytes,
 		 size_t req_bytes_len)
 {
+	uint64_t shabuff[2];
+	size_t i;
+	int status;
+	unsigned char counter;
+
 	/* 10.4.1 Hash_df Process */
 
-#define MAX_NO_OF_BYTES	(255 * DRBG_OUT_LEN)
 	if(!req_bytes_len)
 		return 0;	/* no bytes requested: do nothing */
 	if(!req_bytes || !input)
@@ -568,17 +575,14 @@ int drbg_hash_df(const unsigned char *input,
 	unsigned char temp[len * DRBG_OUT_LEN];
 
 	/* step 3 */
-	unsigned counter = 0x01;
+	counter = 0x01;
 
 	/* step 4 */
 	const size_t _tmp_len = 1 + sizeof(no_of_bits_to_return) + input_len;
 	unsigned char _tmp[_tmp_len];
 	memcpy(_tmp + 1, &no_of_bits_to_return, sizeof(no_of_bits_to_return));
 	memcpy(_tmp + 1 + sizeof(no_of_bits_to_return), input, input_len);
-	int status;
-	uint64_t shabuff[2];
-	size_t i = 1;
-	for(; i <= len; i++){
+	for(i = 1; i <= len; i++){
 		/* step 4.1 */
 		_tmp[0] = counter;
 		status = s390_sha_hw(SHA_512_DEFAULT_IV, _tmp, _tmp_len,
@@ -603,56 +607,12 @@ _exit_:
 	return status;
 }
 
-void drbg_zmem(void *ptr,
-	       size_t len)
-{
-	if(ptr)
-		memset(ptr, 0, len);
-
-	/* protect this code from unwanted compiler optimization */
-	__asm__ __volatile__ ("": :"r"(ptr) :"memory");
-}
-
-int drbg_check_zmem(void *ptr,
-		    size_t len)
-{
-	if(!ptr)
-		return DRBG_HEALTH_TEST_FAIL;
-
-	int i = 0;
-	for(; i < len; i++){
-		if(((unsigned char *)ptr)[i])
-			return DRBG_HEALTH_TEST_FAIL;
-	}
-
-	return 0;
-}
-
-int drbg_mech_valid(const ica_drbg_mech_t *mech)
-{
-	if(!mech)
-		return DRBG_MECH_INV;
-
-	/* Check if @mech is supported. */
-	int i = DRBG_MECH_LIST_LEN - 1;
-	for(; i >= 0; i--){
-		if(DRBG_MECH_LIST[i] == mech)
-			break;
-	}
-	if(i < 0)
-		return DRBG_MECH_INV;
-
-	/* Check if @mech is in error state. */
-	if(mech->error_state)
-		return mech->error_state;
-
-	return 0;
-}
-
 static int test_uninstantiate(ica_drbg_mech_t *mech)
 {
 	/* Error handling test. */
-	int status = drbg_uninstantiate(NULL, false);
+	int status;
+
+	status = drbg_uninstantiate(NULL, false);
 	if(DRBG_SH_INV != status)
 		return DRBG_HEALTH_TEST_FAIL;
 
@@ -660,8 +620,6 @@ static int test_uninstantiate(ica_drbg_mech_t *mech)
 	ica_drbg_t *sh = NULL;
 	status = drbg_instantiate(&sh, mech->highest_supp_sec, true, mech,
 				  NULL, 0, false, NULL, 0, NULL, 0);
-	if(0 > status)
-		mech->error_state = status;
 	if(status)
 		return status;
 
@@ -674,10 +632,12 @@ static int test_uninstantiate(ica_drbg_mech_t *mech)
 
 static int test_instantiate_error_handling(ica_drbg_mech_t *mech)
 {
+	int test_no = 0, status;
+
 	/* Pointer to state handle is NULL. */
-	int test_no = 1;
-	int status = drbg_instantiate(NULL, 0, true, mech, NULL, 0, false,
-				      NULL, 0, NULL, 0);
+	test_no++;
+	status = drbg_instantiate(NULL, 0, true, mech, NULL, 0, false, NULL,
+				  0, NULL, 0);
 	if(DRBG_SH_INV != status)
 		return test_no;
 
@@ -738,9 +698,10 @@ static int test_instantiate_error_handling(ica_drbg_mech_t *mech)
 
 static int test_reseed_error_handling(ica_drbg_mech_t *mech)
 {
+	int test_no = 0, status;
+
 	/* Invalid state handle. */
-	int test_no = 1;
-	int status = drbg_reseed(NULL, true, NULL, 0, false, NULL, 0);
+	status = drbg_reseed(NULL, true, NULL, 0, false, NULL, 0);
 	if(DRBG_SH_INV != status)
 		return test_no;
 
@@ -796,12 +757,13 @@ static int test_generate_error_handling(ica_drbg_mech_t *mech)
 {
 	const int SEC[] = {DRBG_SEC_112, DRBG_SEC_128, DRBG_SEC_192,
 			   DRBG_SEC_256};
+	int test_no = 0, status, i;
+	unsigned char prnd;
 
 	/* Invalid state handle. */
-	int test_no = 1;
-	unsigned char prnd;
-	int status = drbg_generate(NULL, mech->highest_supp_sec, false, NULL,
-				   0, false, NULL, 0, &prnd, sizeof(prnd));
+	test_no++;
+	status = drbg_generate(NULL, mech->highest_supp_sec, false, NULL, 0,
+			       false, NULL, 0, &prnd, sizeof(prnd));
 	if(DRBG_SH_INV != status)
 		return test_no;
 
@@ -867,8 +829,7 @@ static int test_generate_error_handling(ica_drbg_mech_t *mech)
 	test_no++;
 	test_sh.mech = mech;
 	test_sh.sec = mech->highest_supp_sec;
-	int i = 0;
-	for(; i < sizeof(SEC); i++){
+	for(i = 0; i < sizeof(SEC); i++){
 		if(SEC[i] > mech->highest_supp_sec)
 			break;
 		status = drbg_generate(&test_sh, SEC[i], true, NULL, 0, true,
@@ -896,11 +857,26 @@ static int test_generate_error_handling(ica_drbg_mech_t *mech)
 	return 0;
 }
 
-void drbg_recursive_mutex_init(pthread_mutex_t *lock)
+static int set_error_state(ica_drbg_mech_t *mech,
+			   int error)
 {
-	pthread_mutexattr_t attr;
+#ifdef ICA_FIPS
+	fips |= ICA_FIPS_RNG;
 
-	assert(!pthread_mutexattr_init(&attr));
-	assert(!pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE));
-	assert(!pthread_mutex_init(lock, &attr));
+	/* Write to syslog in FIPS-enabled built. The DRBG failure is critical
+	 * here since the old PRNG code is diasbled at compile time. */
+	switch (error) {
+	case DRBG_HEALTH_TEST_FAIL:
+		syslog(LOG_ERR, "Libica DRBG-%s test failed.", mech->id);
+		break;
+	case DRBG_ENTROPY_SOURCE_FAIL:
+		syslog(LOG_ERR, "Libica DRBG-%s entropy source failed.",
+		    mech->id);
+		break;
+	default:
+		break;	/* unreachable */
+	}
+#endif /* ICA_FIPS */
+
+	return mech->error_state = error;
 }
