@@ -17,8 +17,18 @@
 #include <string.h>
 #include <errno.h>
 #include <stdint.h>
+#include <openssl/crypto.h>
 #include <openssl/rsa.h>
+#include <openssl/crypto.h>
+#include <pthread.h>
+#include <semaphore.h>
 
+#include <openssl/opensslconf.h>
+#ifdef OPENSSL_FIPS
+#include <openssl/fips.h>
+#endif /* OPENSSL_FIPS */
+
+#include "fips.h"
 #include "s390_rsa.h"
 #include "s390_prng.h"
 
@@ -41,41 +51,99 @@ static unsigned int mod_expo_sw(int arg_length, char *arg, int exp_length,
 				char *exp, int mod_length, char *mod,
 				int *res_length, char *res, BN_CTX *ctx);
 
-RSA* rsa_key_generate(unsigned int modulus_bit_length,
-		      unsigned long *public_exponent)
+struct thread_data
 {
-	BN_GENCB cb;
+	unsigned int mod_bit_length;
+	unsigned long *pub_exp;
+	RSA     *rsa;
+};
+
+static void *__rsa_key_generate(void *ptr)
+{
+	struct thread_data *pth_data;
+	unsigned int modulus_bit_length;
+	unsigned long *public_exponent;
+
+	pth_data = (struct thread_data*)ptr;
+	modulus_bit_length = pth_data->mod_bit_length;
+	public_exponent = pth_data->pub_exp;
+
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return NULL;
+#endif /* ICA_FIPS */
 
 	if (*public_exponent == 0)
 	{
 		do {
-			s390_prng((unsigned char*)public_exponent,
-				  sizeof(unsigned long));
+			if (s390_prng((unsigned char*)public_exponent,
+			    sizeof(unsigned long)) != 0)
+				return NULL;
 		} while (*public_exponent <= 2 || !(*public_exponent % 2));
 	}
 
-	BIGNUM *exp = BN_new();
 	RSA *rsa = RSA_new();
-
+	BIGNUM *exp = BN_new();
 	if (!exp || !rsa) {
-		if (exp)
-			BN_free(exp);
-		if (rsa)
-			RSA_free(rsa);
-		return 0;
+		RSA_free(rsa);
+		BN_free(exp);
+		return NULL;
 	}
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	BN_GENCB *cb = BN_GENCB_new();
+	if(!cb) {
+		RSA_free(rsa);
+		BN_free(exp);
+		return NULL;
+	}
+#else
+	BN_GENCB dummy;
+	BN_GENCB *cb = &dummy;
+#endif /* OPENSSL_VERSION_NUMBER */
 
 	BN_set_word(exp, *public_exponent);
-	BN_GENCB_set_old(&cb, NULL, NULL);
+	BN_GENCB_set_old(cb, NULL, NULL);
 
-	if (RSA_generate_key_ex(rsa, modulus_bit_length, exp, &cb)) {
-		BN_free(exp);
-		return rsa;
+	if (RSA_generate_key_ex(rsa, modulus_bit_length, exp, cb) == 0) {
+		RSA_free(rsa);
+		rsa = NULL;
+		pth_data->rsa = NULL;
 	}
 
+	BN_free(exp);
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	BN_GENCB_free(cb);
+#endif /* OPENSSL_VERSION_NUMBER */
+	pth_data->rsa = rsa;
 	return 0;
 }
 
+
+RSA* rsa_key_generate(unsigned int modulus_bit_length,
+		      unsigned long *public_exponent)
+{
+	pthread_t tid;
+	struct thread_data th_data;
+	int rc;
+
+	sem_wait(&openssl_crypto_lock_mtx);
+
+	th_data.mod_bit_length = modulus_bit_length;
+	th_data.pub_exp = public_exponent;
+	rc = pthread_create(&(tid), NULL, (void *)&__rsa_key_generate,
+			    (void *)(&th_data));
+	if (rc)
+		return 0;
+	rc = pthread_join(tid, NULL);
+
+	if (!rc && th_data.rsa) {
+		sem_post(&openssl_crypto_lock_mtx);
+		return th_data.rsa;
+	}
+	sem_post(&openssl_crypto_lock_mtx);
+	return NULL;
+}
+
 /**
  * @brief Create a RSA modulus/expo key pair
  *
@@ -98,12 +166,26 @@ unsigned int rsa_key_generate_mod_expo(ica_adapter_handle_t deviceHandle,
 				       ica_rsa_key_mod_expo_t *public_key,
 				       ica_rsa_key_mod_expo_t *private_key)
 {
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	RSA *rsa = rsa_key_generate(modulus_bit_length,
 				    (unsigned long*)(public_key->exponent +
 				    public_key->key_length -
 				    sizeof(unsigned long)));
 	if (!rsa)
 		return errno;
+
+	const BIGNUM *n, *d;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	RSA_get0_key(rsa, &n, NULL,  &d);
+#else
+	n = rsa->n;
+	d = rsa->d;
+#endif /* OPENSSL_VERSION_NUMBER */
+
 	/* Set key buffers zero to make sure there is no
 	 * unneeded junk in between.
 	 */
@@ -111,24 +193,24 @@ unsigned int rsa_key_generate_mod_expo(ica_adapter_handle_t deviceHandle,
 	memset(private_key->modulus, 0, private_key->key_length);
 	memset(private_key->exponent, 0, private_key->key_length);
 
-	unsigned int bn_length = BN_num_bytes(rsa->n);
+	unsigned int bn_length = BN_num_bytes(n);
 	unsigned int offset = 0;
 
 	if (bn_length < public_key->key_length)
 		offset = public_key->key_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->n, public_key->modulus + offset);
+	BN_bn2bin(n, public_key->modulus + offset);
 
 	memcpy(private_key->modulus, public_key->modulus,
 	       public_key->key_length);
 
-	bn_length = BN_num_bytes(rsa->d);
+	bn_length = BN_num_bytes(d);
 	if (bn_length < private_key->key_length)
 		offset = private_key->key_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->d, private_key->exponent + offset);
+	BN_bn2bin(d, private_key->exponent + offset);
 
 	RSA_free(rsa);
 
@@ -154,6 +236,11 @@ unsigned int rsa_key_generate_crt(ica_adapter_handle_t deviceHandle,
 				  ica_rsa_key_mod_expo_t *public_key,
 				  ica_rsa_key_crt_t *private_key)
 {
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	RSA *rsa = rsa_key_generate(modulus_bit_length,
 				    (unsigned long*)(public_key->exponent +
 				    public_key->key_length -
@@ -161,6 +248,20 @@ unsigned int rsa_key_generate_crt(ica_adapter_handle_t deviceHandle,
 	if (!rsa)
 		return errno;
 
+	const BIGNUM *n, *p, *q, *dmp1, *dmq1, *iqmp;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	RSA_get0_key(rsa, &n, NULL,  NULL);
+	RSA_get0_factors(rsa, &p, &q);
+	RSA_get0_crt_params(rsa, &dmp1, &dmq1, &iqmp);
+#else
+	n = rsa->n;
+	p = rsa->p;
+	q = rsa->q;
+	dmp1 = rsa->dmp1;
+	dmq1 = rsa->dmq1;
+	iqmp = rsa->iqmp;
+#endif /* OPENSSL_VERSION_NUMBER */
+
 	/* Public exponent has already been set, no need to do this here.
 	 * For public key, only modulus needs to be set.
 	 */
@@ -169,12 +270,12 @@ unsigned int rsa_key_generate_crt(ica_adapter_handle_t deviceHandle,
 	/* Make sure that key parts are copied to the end of the buffer */
 	unsigned int offset = 0;
 
-	unsigned int bn_length = BN_num_bytes(rsa->n);
+	unsigned int bn_length = BN_num_bytes(n);
 	if (bn_length < public_key->key_length)
 		offset = public_key->key_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->n, public_key->modulus + offset);
+	BN_bn2bin(n, public_key->modulus + offset);
 
 	memset(private_key->p, 0, (private_key->key_length+1) / 2 + 8);
 	memset(private_key->q, 0, (private_key->key_length+1) / 2);
@@ -190,45 +291,45 @@ unsigned int rsa_key_generate_crt(ica_adapter_handle_t deviceHandle,
 	 */
 
 	/* Copy p into buffer */
-	bn_length = BN_num_bytes(rsa->p);
+	bn_length = BN_num_bytes(p);
 	if(bn_length < key_part_length)
 		offset = key_part_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->p, private_key->p + 8 + offset);
+	BN_bn2bin(p, private_key->p + 8 + offset);
 
 	/* Copy q into buffer */
-	bn_length = BN_num_bytes(rsa->q);
+	bn_length = BN_num_bytes(q);
 	if(bn_length < key_part_length)
 		offset = key_part_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->q, private_key->q + offset);
+	BN_bn2bin(q, private_key->q + offset);
 
 	/* Copy dp into buffer */
-	bn_length = BN_num_bytes(rsa->dmp1);
+	bn_length = BN_num_bytes(dmp1);
 	if(bn_length < key_part_length)
 		offset = key_part_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->dmp1, private_key->dp + 8 + offset);
+	BN_bn2bin(dmp1, private_key->dp + 8 + offset);
 
 	/* Copy dq into buffer */
-	bn_length = BN_num_bytes(rsa->dmq1);
+	bn_length = BN_num_bytes(dmq1);
 	if(bn_length < key_part_length)
 		offset = key_part_length - bn_length;
 	else
 		offset = 0;
-	BN_bn2bin(rsa->dmq1, private_key->dq + offset);
+	BN_bn2bin(dmq1, private_key->dq + offset);
 
 	/* Copy qInverse into buffer */
-	bn_length = BN_num_bytes(rsa->iqmp);
+	bn_length = BN_num_bytes(iqmp);
 	if(bn_length < key_part_length)
 		offset = key_part_length - bn_length;
 	else
 		offset = 0;
 
-	BN_bn2bin(rsa->iqmp, private_key->qInverse + 8 + offset);
+	BN_bn2bin(iqmp, private_key->qInverse + 8 + offset);
 
 	RSA_free(rsa);
 
@@ -242,6 +343,11 @@ unsigned int rsa_mod_mult_sw(ica_rsa_modmult_t *pMul)
 	int rc = 0;
 	BN_CTX *ctx = NULL;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	if ((ctx = BN_CTX_new()) == NULL) {
 		return errno;
 	}
@@ -273,6 +379,11 @@ static unsigned int mod_mul_sw(int fc_1_length, char *fc1, int fc_2_length,
 	BIGNUM *b_mod = NULL;
 	BIGNUM *b_res = NULL;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	BN_CTX_start(ctx);
 
 	b_fc1 = BN_CTX_get(ctx);
@@ -336,6 +447,11 @@ unsigned int rsa_mod_expo_sw(ica_rsa_modexpo_t *pMex)
 	int rc = 0;
 	BN_CTX *ctx = NULL;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	if ((ctx = BN_CTX_new()) == NULL) {
 		return errno;
 	}
@@ -395,6 +511,11 @@ static unsigned int mod_expo_sw(int arg_length, char *arg, int exp_length,
 	BN_CTX *mod_expo_ctx = NULL;
 	int mod_expo_rc = 1;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	BN_CTX_start(ctx);
 
 	b_arg = BN_CTX_get(ctx);
@@ -472,7 +593,10 @@ unsigned int rsa_crt_sw(ica_rsa_modexpo_crt_t * pCrt)
 	int short_length = 0;
 	BN_CTX *ctx = NULL;
 
-
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
 
 	short_length = (pCrt->inputdatalength+1) / 2;
 	long_length = short_length + 8;
@@ -612,6 +736,11 @@ static unsigned int mod_sw(int arg_length, char *arg, int mod_length,
 	BIGNUM *b_mod = NULL;
 	BIGNUM *b_res = NULL;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	BN_CTX_start(ctx);
 
 	b_arg = BN_CTX_get(ctx);
@@ -688,6 +817,11 @@ static unsigned int mod_sub_sw(int min_length, char *minu, int sub_length,
 
 	int min_size, sub_size, dif_size;
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	BIGNUM *b_min = NULL;
 	BIGNUM *b_sub = NULL;
 	BIGNUM *b_mod = NULL;
@@ -789,6 +923,11 @@ static unsigned int add_sw(int aug_length, char *aug, int add_length,
 
 	BN_CTX_start(ctx);
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	b_aug = BN_CTX_get(ctx);
 	b_add = BN_CTX_get(ctx);
 	if ((b_res = BN_CTX_get(ctx)) == NULL) {
@@ -853,6 +992,11 @@ static unsigned int mul_sw(int fc_1_length, char *fc1, int fc_2_length,
 
 	BN_CTX_start(ctx);
 
+#ifdef ICA_FIPS
+	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+		return EACCES;
+#endif /* ICA_FIPS */
+
 	b_fc1 = BN_CTX_get(ctx);
 	b_fc2 = BN_CTX_get(ctx);
 	if ((b_res = BN_CTX_get(ctx)) == NULL) {
